package com.wistone.pay.net;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.http.HttpStatus;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONObject;

import android.os.Handler;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.Log;

import com.wistone.pay.net.command.Command;
import com.wistone.pay.net.data.CommandConst;
import com.wistone.pay.net.http.CommandHttpClient;
import com.wistone.pay.net.http.CommandHttpResult;

/**************************************************************************
 * CommandUtil</br>
 * Author : isUseful ? TanJian : Unknown</br>
 * English by google translate.
 **************************************************************************/
public class CommandUtil
{
	// 发送完成消息
	private static final int	WHAT_COMMANDFINISH	= /* Tel: */1348/* 8776888 */;
	private static final String	COMMAND_POST_KEY	= "protocol";

	private static Object		lock				= new Object();
	private static CommandUtil	instance;
	private ExecutorService		executor			= null;

	private String				paymentServerUrl;

	private CommandUtil()
	{
		executor = Executors.newCachedThreadPool();
		paymentServerUrl = CommandConst.URL;
	}

	public static CommandUtil getInstance()
	{
		if (instance == null)
		{
			synchronized (lock)
			{
				if (instance == null)
				{
					instance = new CommandUtil();
				}
			}
		}
		return instance;
	}

	public final void setPaymentServerUrl(String url)
	{
		paymentServerUrl = url;
	}

	public final void sendCommand(CommandCallback callback, Command command)
	{
		if (command == null)
		{
			throw new NullPointerException("Command NOT BE NULL!");
		}
		if (TextUtils.isEmpty(command.getUrl()))
		{
			throw new NullPointerException("Command Url NOT BE NULL!");
		}
		Handler handler = new SendCommandHandler(callback);
		executor.execute(new SendCommandThread(command, handler));
	}

	class SendCommandHandler extends Handler
	{
		private CommandCallback	pCallBack;

		public SendCommandHandler(CommandCallback pCallBack)
		{
			this.pCallBack = pCallBack;
		}

		@Override
		public void handleMessage(Message message)
		{
			int what = message.what;
			switch (what)
			{
				case WHAT_COMMANDFINISH:
					if (pCallBack != null)
					{
						pCallBack.onCommandFinished((Command) message.obj);
					}
					break;
			}
		}
	}

	class SendCommandThread implements Runnable
	{
		private Command	command;
		private Handler	handler;

		public SendCommandThread(Command command, Handler handler)
		{
			this.command = command;
			this.handler = handler;
		}

		@Override
		public void run()
		{
			if (command == null || handler == null)
			{
				return;
			}

			long startTime = SystemClock.uptimeMillis();
			command = doSendCommand(command);
			long endTime = SystemClock.uptimeMillis();

			Log.i(CommandConst.TAG, "connect used time :" + (endTime - startTime) + "ms");

			Message message = handler.obtainMessage(WHAT_COMMANDFINISH, command);
			handler.sendMessage(message);
		}

		private Command doSendCommand(Command command)
		{
			try
			{
				CommandHttpResult result;
				String url = paymentServerUrl + command.getUrl();
				JSONObject paramsJson = command.getCommand();
				if (paramsJson == null)
				{
					result = CommandHttpClient.post(url);
				}
				else
				{
					// String encodeString = URLEncoder.encode(paramsJson.toString(), HTTP.UTF_8);
					BasicNameValuePair pair = new BasicNameValuePair(COMMAND_POST_KEY, paramsJson.toString());
					result = CommandHttpClient.post(url, pair);
				}
				if (result == null || result.getStatusCode() != HttpStatus.SC_OK)
				{
					command.stateCode = Command.COMMAND_CLIENT_ERROR;
				}
				else
				{
					command.doCommand(result.getResult());
				}
			}
			catch (Exception e)
			{
				Log.i(CommandConst.TAG, e.toString());
				command.stateCode = Command.COMMAND_CLIENT_ERROR;
			}
			return command;
		}
	}
}
